<?php

    class Send_NRDP_Action extends Reactor_Action
    {
        protected $tpl = 'send_nrdp.php';
        protected $name = 'Send an NRDP check result';
		
		public static $registration = array(
			ACTION_NAME => 'Send NRDP',
			ACTION_CATEGORY => 'Network',
			ACTION_DESCRIPTION => 'Send an NRDP check result to a Nagios NRDP server',
			ACTION_AUTHOR => 'Mike Guthrie, Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '03/12/2013', //Last version update
			ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			ACTION_TYPE => 'core', //core | user. core modules can't be deleted
			ACTION_META => array(),	//Misc information	
		); 

        protected $fields = array(
            'url' => array(
                'label' => 'url',
                'validate' => 'trim|required|callback_check_nrdp_url|htmlentities'
            ),		
            'token' => array(
                'label' => 'token',
                'validate' => 'trim|required'
            ),
            'host' => array(
                'label' => 'host',
                'validate' => 'trim|xss_clean|required'
            ),			
            'service' => array(
                'label' => 'service',
                'validate' => 'trim|xss_clean'
            ),			
            'state' => array(
                'label' => 'state',
                'validate' => 'trim|numeric'
            ),	
            'output' => array(
                'label' => 'output',
                'validate' => 'trim'
            ),			

        );
		
        public function check_nrdp_url($str)
        {

            $source = $this->ci->input->post('source');
                if(substr($str, -1) != "/"){
                    $this->ci->form_validation->set_message('check_nrdp_url', _('The trailing slash is required.'));
                    return FALSE;
                }
                    
            return TRUE;
        }
     
        public function get_label()
        {
			if($params = $this->get_params()){
			
				//$state="STATE";
				//$host="HOST";
				//$service="SERVICE";
				$state=$params['state'];
				$host=$params['host'];
				$service=$params['service'];
				
				switch($state){
					case 0:
						if($service!="")
							$sstr="OK";
						else
							$sstr="UP";
						break;
					case 1:
						if($service!="")
							$sstr="WARNING";
						else
							$sstr="DOWN";
						break;
					case 2:
						if($service!="")
							$sstr="CRITICAL";
						else
							$sstr="UNREACHABLE";
						break;
					case 3:
						$sstr="UNKNOWN";
						break;
					default:
						$sstr="?";
					}
				$ostr="?";
				if($service!="")
					$ostr="service '".$service."' on host '".$host."'";
				else
					$ostr="host '".$host."'";
					
				if(isset($params['url'])) {
					return sprintf(_('Send NRDP check result of %s for %s to \'%s\''),$sstr,$ostr, htmlentities($params['url']));
					}
				}
            return parent::get_label();
        }

        public function run($context)
        {
            $this->set_context($context);
            $params = $this->get_params();
            $url = $this->get_param('url');
			$service =  $this->get_param('service');


			
			$type = empty($service) ? 'host' : 'service';
			$servicexml = empty($service) ? '' : "<servicename>".htmlentities($service)."</servicename>"; 
			
			$xml=
			"<?xml version='1.0'?> 
			<checkresults>
				<checkresult type='{$type}'>
					<hostname>".htmlentities($this->get_param('host'))."</hostname>
					{$servicexml}
					<state>".$this->get_param('state')."</state>
					<output>".$this->get_param('output')."</output>
				</checkresult>
			</checkresults>
			";			
	
			//use curl to send data with NRDP
			$args = array(
				'cmd' => 'submitcheck',
				'token' =>  $this->get_param('token'),
				'XMLDATA' => $xml,
			);	
	
			$argstring = http_build_query($args);
			$url.='?'.$argstring;
			
            list($http_result, $http_info) = do_curl_request($url,false);
            $this->log('info', 'HTTP status was '.$http_info['http_code']);
            $s = $http_info['http_code'];
            $this->set_result(($s >= 200 && $s < 300) ? 0 : 1);
			
			if(ENVIRONMENT=='development')
				logit("NRDP Result: ".print_r($http_result,true));
            
            return $context;
        }
    	
	
}	
	
	
